% Fractional Fast Fourier Transform for the Black Scholes Model
% --------------------------------------
% Inputs
%   N  = number of discretization points
%   uplimit = Upper limit of integration
%   S0 = spot price
%   r = risk free rate
%   tau = maturity
%   sigma = volatility
%   alpha = dampening factor
%   fast = fast versus slow algorithm.
%     fast = 1 fast version.  Uses vectorization.
%     fast = 0 slow version.  Uses loops
% --------------------------------------
% Outputs
%   CallFFT = Black Scholes call prices using FFT
%   CallBS  = Black Scholes call prices using closed form
%         K = Strike prices
% --------------------------------------
% By Fabrice Douglas Rouah www.Volopta.com
 
clc; clear;

%% Inline Functions
% Black-Scholes call price
BSC = inline('s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T))',...
	         's','K','r','q','v','T');
		 
% Black-Scholes characteristic function for ln(S(T))
BSCF = inline('exp(i*(log(S0) + (r-sigma^2/2)*T)*phi - 0.5*sigma^2*T*phi^2)',...
              'phi','S0','r','sigma','T');

%% Required inputs
S0 = 100;           % Spot price.
r = 0.04;           % Risk free rate.
q = 0.01;           % Dividend yield
tau = .25;          % Time to maturity.
sigma = 0.5;        % Volatility
alpha = 1.75;       % Dampening factor
N = 2^13;           % Grid size
uplimit = 500;      % Upper limit of integration
fast = 1;           % Choice of fast (1) or slow (0) algorithm

%% Set up the FFT for y and z with M=2*N points
% Specify the weights
M = 2*N;
w = ones(M,1);
w(1) = 1/2;
w(M) = 1/2;

% Specify the increments
eta = .001;             % integration range
lambdainc = eta;        % strike range

% Specify the b parameter
b = M*lambdainc/2;

% Create the grid for the integration, using M points
v = eta.*[0:M-1]';

% Create the grid for the log-strikes, using M points
s0 = log(S0);
k = -b + lambdainc.*[0:M-1]' + s0;
K = exp(k);

% Create the vector x(1),...,x(M)
if fast==1
	% Implement the FFT - fast algorithm
	U = [0:M-1];
	J = [0:M-1];
	psi = BlackScholesCF(v-(alpha+1).*i,S0,r,sigma,tau);
	psi = conj(psi);
	phi = exp(-r*tau).*psi./conj(alpha.^2 + alpha - v.^2 + i.*v.*(2*alpha+1));
	d = conj(alpha.^2 + alpha - v.^2 + i.*v.*(2*alpha+1));
	x = conj(exp(i.*(b-s0).*v)).*phi.*w;
	x = x';
elseif fast==0
	% Implement the FFT - slow algorithm
	for u=1:M
		for j=1:M
			psi(j) = BSCF(v(j)-(alpha+1)*i,S0,r,sigma,tau);
			phi(j) = exp(-r*tau)*psi(j)/(alpha^2 + alpha - v(j)^2 + i*v(j)*(2*alpha+1));
			x(j) = exp(i*(b-s0)*v(j))*phi(j)*w(j);
		end
	end
end

%% Create the FFT for y
beta = eta*lambdainc/2/pi;

% Apply the FRFT on the vector x
Call = FRFT(x,beta)';

% Retain only the first N values, discard the last N values
I = (1:N);
Call = Call(I);

% Produce the call price
CallFRFT = real(Call).*exp(-alpha.*k(I)).*eta./pi;

%% Obtain the Black-Scholes call price using the closed form and plot
CallBS = BlackScholesCall(S0,K(I),r,q,sigma,tau);

% Plot the results
plot(K(I), CallFRFT, K(I), CallBS)
legend('BS Call Price from FRFT', 'BS Call Price from Closed Form')
